@@ -40,8 +40,15 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define LND_TCP_MSL                  60
 #define LND_TCP_TIMEOUT              5*60
+#define LND_SETUP_TIMEOUT            75 /* See Stevens TCP Ill Vol 1 18.3 */
 #define LND_TCP_TOTAL_TIMEOUT        7500 /* Bit over two hours -- no TCP survives that */
-#define LND_CONN_GENERIC_TIMEOUT     5*60 /* 5 minutes */
+#define LND_CONN_GENERIC_TIMEOUT     1*60 /* 1 minute */
+
+int libnd_conntrack_tcp_msl = LND_TCP_MSL;
+int libnd_conntrack_tcp_timeout = LND_TCP_TIMEOUT;
+int libnd_conntrack_tcp_setup_timeout = LND_TCP_TIMEOUT;
+int libnd_conntrack_tcp_total_timeout = LND_TCP_TOTAL_TIMEOUT;
+int libnd_conntrack_generic_timeout = LND_CONN_GENERIC_TIMEOUT;
 
 struct lnd_tcp_conn
 {
@@ -238,7 +245,7 @@ libnd_tcpconn_is_timeout(LND_TCPConn *tcpc, const LND_Packet *packet)
 
   pcapnav_timeval_sub(&packet->ph.ts, &conn->latest_ts, &diff);
 	
-  if (diff.tv_sec >= LND_TCP_TOTAL_TIMEOUT)
+  if (diff.tv_sec >= libnd_conntrack_tcp_total_timeout)
     {
       tcpc->state = LND_TCP_CLOSED_TIMEOUT;
       return TRUE;
@@ -248,7 +255,7 @@ libnd_tcpconn_is_timeout(LND_TCPConn *tcpc, const LND_Packet *packet)
 
   if (tcpc->state == LND_TCP_SYN_SENT)
     {      
-      if (diff.tv_sec >= LND_TCP_TIMEOUT)
+      if (diff.tv_sec >= libnd_conntrack_tcp_setup_timeout)
 	{
 	  tcpc->state = LND_TCP_CLOSED_TIMEOUT;
 	  return TRUE;
@@ -257,7 +264,7 @@ libnd_tcpconn_is_timeout(LND_TCPConn *tcpc, const LND_Packet *packet)
   
   if (tcpc->state == LND_TCP_SHUTDOWN)
     {
-      if (diff.tv_sec >= LND_TCP_TIMEOUT)
+      if (diff.tv_sec >= libnd_conntrack_tcp_timeout)
 	{
 	  tcpc->state = LND_TCP_CLOSED_TIMEOUT;
 	  return TRUE;
@@ -268,7 +275,7 @@ libnd_tcpconn_is_timeout(LND_TCPConn *tcpc, const LND_Packet *packet)
 
   if (tcpc->state == LND_TCP_TIME_WAIT)
     {        
-      if (diff.tv_sec >= 2 * LND_TCP_MSL)
+      if (diff.tv_sec >= 2 * libnd_conntrack_tcp_msl)
 	{
 	  tcpc->state = LND_TCP_CLOSED_NORMAL;
 	  return TRUE;
@@ -277,7 +284,7 @@ libnd_tcpconn_is_timeout(LND_TCPConn *tcpc, const LND_Packet *packet)
   
   if (tcpc->state == LND_TCP_RST_WAIT)
     {
-      if (diff.tv_sec >= LND_TCP_MSL)
+      if (diff.tv_sec >= libnd_conntrack_tcp_msl)
 	{
 	  tcpc->state = LND_TCP_CLOSED_RST;
 	  return TRUE;
@@ -311,7 +318,7 @@ libnd_tcpconn_update_time(LND_TCPConn *tcpc, const struct bpf_timeval *tv)
    */
   pcapnav_timeval_sub(tv, &conn->latest_ts, &diff);
 	
-  if (diff.tv_sec >= LND_TCP_TOTAL_TIMEOUT)
+  if (diff.tv_sec >= libnd_conntrack_tcp_total_timeout)
     {
       tcpc->state = LND_TCP_CLOSED_TIMEOUT;
       return;
@@ -322,35 +329,35 @@ libnd_tcpconn_update_time(LND_TCPConn *tcpc, const struct bpf_timeval *tv)
     case LND_TCP_SYN_SENT:
       pcapnav_timeval_sub(tv, &tcpc->timeout_ts, &diff);
       
-      if (diff.tv_sec >= LND_TCP_TIMEOUT)
+      if (diff.tv_sec >= libnd_conntrack_tcp_timeout)
 	tcpc->state = LND_TCP_CLOSED_TIMEOUT;
       break;
 
     case LND_TCP_SYN_ACK_SENT:
       pcapnav_timeval_sub(tv, &tcpc->timeout_ts, &diff);
       
-      if (diff.tv_sec >= LND_TCP_TIMEOUT)
+      if (diff.tv_sec >= libnd_conntrack_tcp_timeout)
 	tcpc->state = LND_TCP_CLOSED_TIMEOUT;
       break;
       
     case LND_TCP_SHUTDOWN:
       pcapnav_timeval_sub(tv, &tcpc->timeout_ts, &diff);
       
-      if (diff.tv_sec >= LND_TCP_TIMEOUT)
+      if (diff.tv_sec >= libnd_conntrack_tcp_timeout)
 	tcpc->state = LND_TCP_CLOSED_TIMEOUT;
       break;
 
     case LND_TCP_TIME_WAIT:
       pcapnav_timeval_sub(tv, &tcpc->wait_ts, &diff);
       
-      if (diff.tv_sec >= 2 * LND_TCP_MSL)
+      if (diff.tv_sec >= 2 * libnd_conntrack_tcp_msl)
 	tcpc->state = LND_TCP_CLOSED_NORMAL;	
       break;
 
     case LND_TCP_RST_WAIT:
       pcapnav_timeval_sub(tv, &tcpc->wait_ts, &diff);
       
-      if (diff.tv_sec >= LND_TCP_MSL)
+      if (diff.tv_sec >= libnd_conntrack_tcp_msl)
 	tcpc->state = LND_TCP_CLOSED_RST;
       break;
       
@@ -367,6 +374,7 @@ libnd_tcpconn_update(LND_TCPConn *tcpc,
   struct ip *iphdr;
   struct tcphdr *tcphdr;  
   gboolean forward_flow;
+  LND_ConnID *conn = (LND_ConnID *) tcpc;
 
   if (!tcpc || !packet)
     return;
@@ -378,6 +386,11 @@ libnd_tcpconn_update(LND_TCPConn *tcpc,
   
   forward_flow = (iphdr->ip_src.s_addr == tcpc->conn.ip_src.s_addr);
   
+  if (forward_flow)
+    conn->content_tx += libnd_tcp_get_payload_length(iphdr, tcphdr);
+  else
+    conn->content_rx += libnd_tcp_get_payload_length(iphdr, tcphdr);
+  
   if ((tcphdr->th_flags & TH_RST) &&
       tcpc->state != LND_TCP_RST_WAIT)      
     {
@@ -529,6 +542,29 @@ libnd_tcpconn_update(LND_TCPConn *tcpc,
 }
 
 
+void            
+libnd_udpconn_update(LND_ConnID *conn,
+		     const LND_Packet *packet)
+{
+  struct ip *iphdr;
+  struct udphdr *udphdr;  
+  gboolean forward_flow;
+
+  if (!conn || !packet)
+    return;
+
+  if (! libnd_udp_get_headers(packet, &iphdr, &udphdr))
+    return;
+  
+  forward_flow = (iphdr->ip_src.s_addr == conn->ip_src.s_addr);
+  
+  if (forward_flow)
+    conn->content_tx += libnd_udp_get_payload_length(iphdr, udphdr);
+  else
+    conn->content_rx += libnd_udp_get_payload_length(iphdr, udphdr);
+}
+
+
 gboolean
 libnd_conn_get_packet_dir(const LND_ConnID *conn, const LND_Packet *packet)
 {
@@ -570,7 +606,7 @@ libnd_conn_is_dead(LND_ConnID *conn, const LND_Packet *packet)
 	{
 	  pcapnav_timeval_sub(&packet->ph.ts, &conn->latest_ts, &tv_diff);
 	    
-	  if (tv_diff.tv_sec >= LND_CONN_GENERIC_TIMEOUT)
+	  if (tv_diff.tv_sec >= libnd_conntrack_generic_timeout)
 	    /* He's dead, Jim. */
 	    return TRUE;
 	}
@@ -645,8 +681,11 @@ libnd_conn_new(const LND_Packet *packet)
   LND_ProtoData *proto_data;
   struct ip *iphdr;
 
-   if (! (proto_data = libnd_packet_get_proto_data(packet, libnd_ip_get(), 0)))
-     return NULL;
+  if (! packet)
+    return NULL;
+  
+  if (! (proto_data = libnd_packet_get_proto_data(packet, libnd_ip_get(), 0)))
+    return NULL;
 
   iphdr = (struct ip*) proto_data->data;
 
@@ -709,6 +748,10 @@ libnd_conn_update(LND_ConnID *conn,
       libnd_tcpconn_update((LND_TCPConn *) conn, packet);
       break;
       
+    case IPPROTO_UDP:
+      libnd_udpconn_update(conn, packet);
+      break;
+
     default:
       /* Currently nothing */
       break;
